@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 Vincenzo Arcidiacono;
+# Copyright 2020-2022 Vincenzo Arcidiacono;
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -122,8 +122,8 @@ def save_json(output_fpath, data):
         File path where output are written.
     :rtype: str
     """
-    import json
+    import simplejson as json
     os.makedirs(osp.dirname(output_fpath) or '.', exist_ok=True)
     with open(output_fpath, 'w') as file:
-        json.dump(data, file, default=_json_default)
+        json.dump(data, file, default=_json_default, ignore_nan=True)
     return output_fpath
